-------------------------------------------------------------
--	Filename:  DAC_TEST.VHD
-- Author: Adam Kwiatkowski / MSS
--	Version: 1
--	Date last modified: 11/29/05
-- Inheritance: 	none
--
-- Description: simple ramp function to test DAC controller
-- Rev 1. 11/29/05 AK 
-- enable feature added to tri-state DAC
---------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity DAC_TEST is port ( 

	--// Clock
	CLK : in std_logic;		-- Main internal processing clock

	--// User Interface
	ENABLE_N : in std_logic;		-- test enable	active low

	TPS: out std_logic_vector(10 downto 1);		-- test points
	
	--// DAC interface
	SYNC_B: out std_logic;
	SCLK: out std_logic;	
		-- Maximum frequency of 30 MHz.
		-- doesn't toggle when DAC is inactive
	SDIN: out std_logic

);
end entity;

architecture behavioral of DAC_TEST is
--------------------------------------------------------
--      COMPONENTS
--------------------------------------------------------
component DAC is port ( 

	--// Clock
	CLK : in std_logic;		-- Main internal processing clock
	
	--// User Interface
	DATA_IN: in std_logic_vector(9 downto 0);
		-- aligned with DATA_IN_CLK
	DATA_IN_CLK: in std_logic;
		-- 1 CLK pulse indicating that DATA_IN is valid.

	OP_MODE: in std_logic_vector(1 downto 0);
		-- Operation Mode aligned with DATA_IN_CLK
		-- 0 0 - Normal operation
		-- Power-down modes:  
		-- 0 1 - 1 kOhm to GND
		-- 1 0 - 100 kOhm to GND
		-- 1 1 - Three-state

	DONE: out std_logic;
		-- indicates DAC availability	(1 CLK early)
		-- '1' indicates interface is available on NEXT clk. 
		-- goes to '0' immediately after valid DATA_IN_CLK pulse/level
		-- is presented, used for flow control

	--// DAC interface
	SYNC_B: out std_logic;
	SCLK: out std_logic;	
		-- Maximum frequency of 30 MHz.
		-- doesn't toggle when DAC is inactive
	SDIN: out std_logic
);
end component;


--------------------------------------------------------
--     SIGNALS
--------------------------------------------------------

--// Constants
signal ZERO: std_logic;
signal ONE: std_logic;
signal ZERO8: std_logic_vector(7 downto 0);

signal DATA_COUNTER: std_logic_vector(9 downto 0) := (others => '0');
signal DAC_READY: std_logic;
signal SAMPLE_CLK: std_logic;

signal CNT_FLAG: std_logic;

--------------------------------------------------------
--      IMPLEMENTATION
--------------------------------------------------------

begin
--// CONSTANTS ---------------------
ZERO <= '0';
ONE <= '1';
ZERO8 <= (others => '0');

DAC_001: DAC port map(		
	CLK => CLK,
	--// User Interface
	DATA_IN => DATA_COUNTER,
	DATA_IN_CLK => SAMPLE_CLK,
--	OP_MODE => ZERO8(1 downto 0),
	OP_MODE(0) => ENABLE_N,
	OP_MODE(1) => ENABLE_N,
	DONE => DAC_READY,
  	--// DAC interface
  	SYNC_B => SYNC_B,
  	SCLK => SCLK,
  	SDIN => SDIN
);		 

MAIN_001: process(CLK)
begin
	if rising_edge(CLK) then
		if((CNT_FLAG = '1') and (DAC_READY = '1')) then
			-- if DAC driver is available, send new sample
			-- increment DATA_COUNTER only once per DAC_READY pulse
			SAMPLE_CLK <= '1';
			DATA_COUNTER  <= DATA_COUNTER + 1;	
			CNT_FLAG <= '0';	 
		else
			CNT_FLAG <= '1';
			SAMPLE_CLK <= '0';
		end if;						
	end if;						
end process;

--TPS(1) <= DATA_COUNTER(9);
TPS(1) <= DAC_READY;

end behavioral;

