-------------------------------------------------------------
-- Filename:  COM1400.VHD
-- Authors: 
--		Adam Kwiatkowski / MSS
--		Alain Zarembowitch / MSS
-- Version: Rev 2
-- Last modified: 10-5-05
-- Inheritance: 	COM1300C rev12, COM-5002_035_R006
--
-- description:  Top level for COM-1400 template.
-- Includes test points for all four system clocks: 60 MHz input reference,
-- 40 MHz I/O clock, 90 MHz processing clock.
--
-- Size: FFs: 3018, LUT4s 3290, Slices 2672, RAMBs 7, MULT18x18s 5, 
-- GCLKs 8, DCMs 2. 
-- Uses COM-5002.
--
-- Rev 2. 8-29-05 AZ
-- latest USB20.NGC file dated 8/8/05
-- enabled serial connection to bottom connector
-- removed TCP-IP component for clarity and consistency with specifications.
-- general code cleaning for consistency with specifications and comblock control center.
-- Rev 4. 9-29-05 AK
-- general IO test of 40 pin connectors added
-- Rev 6. 10-4-05 AK
-- added error count test point (tied to SREG4-7)
-- added input timing constraint
-- Rev 7. 10-4-05 AK
-- added difference output when not expected value (tied to SREG0-3)
-- 
---------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
-- UNISIM library includes all Xilinx primitives (no longer any need to 
-- declare under the Components section. ISE 6.3.
Library UNISIM;
use UNISIM.vcomponents.all;

entity COM1400 is
    Port ( 
		--GLOBAL CLOCKS
	   USB_CLK60 : in std_logic;	
			-- 60 MHz reference clock. Generated by the USB PHY on this board.

		--// Left connector, 40 pin female 2mm connector. 
		-- Pin 1 is reserved for input clock 
		-- Pins 10,20,30,40 reserved for Ground
		-- Pins 37,38,39 reserved for JTAG
		-- Pins 35-36 reserved for ComBlock Monitoring and Control.
		-- pullup pin B13 for interface with DACs.
		LEFT_CONNECTOR: inout std_logic_vector(34 downto 1);

		--// Right/Bottom connectors: 40 pin male 2mm connector. 
		-- Pin 1 is reserved to forward clock to following module.
		-- Pins 10,20,30,40 reserved for Ground
		-- Pins 37,38,39 reserved for JTAG
		RIGHT_CONNECTOR: inout std_logic_vector(34 downto 1);
		BOTTOM_CONNECTOR: inout std_logic_vector(34 downto 1);

		--// USB PHY interface (SMSC GT3200 IC)
		-- Direct connection between the USB20 component and the USB PHY.
		-- Synchronous with USB_CLK60G clock
		USB_RESET: out std_logic;
--		USB_DATABUS16_8: out std_logic;
--		USB_SUSPENDN: out std_logic;
		USB_XCVRSELECT: out std_logic;
		USB_TERMSELECT: out std_logic;
		USB_OPMODE: out std_logic_vector(1 downto 0);
			-- operational mode
			-- 00 = normal operation
			-- 01 = non-driving (all terminations removed)
			-- 10 = disable bit stuffing and NRZI encoding (unused)
			-- 11 = reserved (unused)
		USB_LINESTATE: in std_logic_vector(1 downto 0);	
			--* possible race condition with USB_CLK60G. Reclock at falling edge.

		USB_TXVALID: out std_logic;
		USB_TXREADY: in std_logic;	
			--* possible race condition with USB_CLK60G. Reclock at falling edge.
--		USB_VALIDH: inout std_logic;	-- VALIDH is not used in 8-bit mode
		USB_RXVALID: in std_logic;	
			--* possible race condition with USB_CLK60G. Reclock at falling edge.
		USB_RXACTIVE: in std_logic;	
			--* possible race condition with USB_CLK60G. Reclock at falling edge.
		USB_RXERROR: in std_logic;	

		USB_DATA_IN: in std_logic_vector(7 downto 0);	
		USB_DATA_OUT: out std_logic_vector(7 downto 0);	

		USB_VBUS_SENSE: in std_logic;
	
		--// DAC interface
		SYNC_B: out std_logic;
		SCLK: out std_logic;
		SDIN: out std_logic;

		--// Monitoring & Control: Atmel controller interface
		UC_AD: inout std_logic_vector(7 downto 0);
		UC_WRN_IN: in std_logic;
		UC_RDN: in std_logic;
		UC_ALE_IN: in std_logic;
		UC_FLASH_CSN: in std_logic;
			-- '0' when uC talks to the flash memory, '1' when otherwise (i.e. possibly
			-- talk to the FPGAs).

		--// Monitoring & Control: Serial ports interface
		-- _L for left connector, _B for bottom connector, _R for right connector.
		-- RX_ signals must be pulled up to signal inactivity on the interface.
		RX_L: in std_logic;
		RX_B: in std_logic;	
		RX_R: in std_logic;	
		TX_L: out std_logic;
		TX_B: out std_logic;	
		TX_R: out std_logic; 

		SERIAL_COM_1 : in std_logic;
		SERIAL_COM_2 : in std_logic;
		--// Test Points
		TEST_POINTS: out std_logic_vector(10 downto 1);
			-- Test points are under the shield. 6 at the edge connector.
		INITB: out std_logic;
			-- easy access test point: INIT#. Easy access with scope.
		DUMMY_OUT: out std_logic
			-- not physically connected to anything
			-- used to prevent alternate function on FPGA pins
			  
			  );
end entity;

architecture Behavioral of COM1400 is
--------------------------------------------------------
--      COMPONENTS
--------------------------------------------------------

component USB20 is
    port ( 
	 	--// Clocks Resets
		ASYNC_RESET: in std_logic;
		USB_CLK60G: in std_logic;	
			-- reference clock. 60 MHz.
 		   -- Supplied by the SMSC GT3200 IC (CLK0UT)
			-- Generally not used outside of this component.
			-- Global clock (BUFG MUST be instantiated outside).
		CLK_P: in std_logic;
			-- Main processing or I/O clock used outside of this component.
			-- All application interface signals are synchronous with CLK_p
			-- Key assumption: CLK_P is slightly faster than USB_CLK60G/2.
			-- Other key assumption: CLK_P < 4 * CLK60G

		--// USB PHY interface (SMSC GT3200 IC)
		-- Direct connection between the USB20 component and the USB PHY.
		-- Synchronous with USB_CLK60G clock
		USB_RESET: out std_logic;
		USB_DATABUS16_8: out std_logic;
		USB_SUSPENDN: out std_logic;
		USB_XCVRSELECT: out std_logic;
		USB_TERMSELECT: out std_logic;
		USB_OPMODE: out std_logic_vector(1 downto 0);
			-- operational mode
			-- 00 = normal operation
			-- 01 = non-driving (all terminations removed)
			-- 10 = disable bit stuffing and NRZI encoding (unused)
			-- 11 = reserved (unused)
		USB_LINESTATE: in std_logic_vector(1 downto 0);	
			--* possible race condition with USB_CLK60G. Reclock at falling edge.

		USB_TXVALID: out std_logic;
		USB_TXREADY: in std_logic;	
			--* possible race condition with USB_CLK60G. Reclock at falling edge.
--		USB_VALIDH: inout std_logic;	-- VALIDH is not used in 8-bit mode
		USB_RXVALID: in std_logic;	
			--* possible race condition with USB_CLK60G. Reclock at falling edge.
		USB_RXACTIVE: in std_logic;	
			--* possible race condition with USB_CLK60G. Reclock at falling edge.
		USB_RXERROR: in std_logic;	-- unused

		USB_DATA_IN: in std_logic_vector(7 downto 0);	
			--* possible race condition with USB_CLK60G. Reclock at falling edge.
		USB_DATA_OUT: out std_logic_vector(7 downto 0);	
			-- time critical. User should add OFFSET OUT contraints in the constraint
			-- editor. 

		USB_VBUS_SENSE: in std_logic;
			

		--// Data Stream 1
		-- Synchronous with CLK_P clock
		DATA1_OUT: out std_logic_vector(7 downto 0);
		DATA1_OUT_SAMPLE_CLK: out std_logic;
			-- read DATA1_OUT at rising edge of CLK_P when DATA1_OUT_SAMPLE_CLK = '1'
			-- The latency from a DATA1_OUT_SAMPLE_CLK_REQ input pulse to the DATA1_OUT_SAMPLE_CLK
			-- output pulse is 2 CLK_P.
		DATA1_OUT_BUFFER_EMPTY: out std_logic;
		DATA1_OUT_SAMPLE_CLK_REQ: in std_logic;	
			-- requests data. If no data is available in the buffer, the
			-- DATA1_OUT_SAMPLE_CLK will stay low.
			-- (flow control)
		
		DATA1_IN: in std_logic_vector(7 downto 0);
		DATA1_IN_SAMPLE_CLK: in std_logic;
			-- read DATA1_IN at rising edge of CLK_P when DATA1_IN_SAMPLE_CLK = '1'
		DATA1_IN_SAMPLE_CLK_REQ: out std_logic;
			-- requests data when the input elastic buffer is less than half full. 
			-- (flow control)

		--// Data Stream 2
		-- Synchronous with CLK_P clock
		DATA2_OUT: out std_logic_vector(7 downto 0);
		DATA2_OUT_SAMPLE_CLK: out std_logic;
			-- read DATA2_OUT at rising edge of CLK_P when DATA2_OUT_SAMPLE_CLK = '1'
		DATA2_OUT_BUFFER_EMPTY: out std_logic;
		DATA2_OUT_SAMPLE_CLK_REQ: in std_logic;	
			-- requests data. If no data is available in the buffer, the
			-- DATA2_OUT_SAMPLE_CLK will stay low.
			-- (flow control)
		
		DATA2_IN: in std_logic_vector(7 downto 0);
		DATA2_IN_SAMPLE_CLK: in std_logic;
			-- read DATA2_IN at rising edge of CLK_P when DATA2_IN_SAMPLE_CLK = '1'
		DATA2_IN_SAMPLE_CLK_REQ: out std_logic;
			-- requests data when the input elastic buffer is less than half full. 
			-- (flow control)


		--// Test Points
		USB_TP: out std_logic_vector(10 downto 1)
			-- bit 1: speed after auto-negotiation with host PC: '1' if high-speed.
			-- bit 2: speed after auto-negotiation with host PC: '1' if full-speed.
			-- bit 3: valid SETUP message (PID valid, CRC5 valid). 
			-- 		SETUP is the first message from the host PC to this USB peripheral
			-- bit 4: Host asks to read the descriptor table.
			-- bit 5: data stream 2 input, elastic buffer write pointer LSb (address bit 0)
			-- bit 6: data stream 2 input, elastic buffer read pointer LSb  (address bit 0)
			-- bit 7: data stream 2 input, elastic buffer write pointer MSb (address bit 10)
			-- bit 8: data stream 2 input, elastic buffer read pointer MSb  (address bit 10)
				-- useful in checking flow control
			-- bit 9: data stream 2 output, elastic buffer write pointer MSb (address bit 10)
			-- bit 10: data stream 2 output, elastic buffer read pointer MSb  (address bit 10)
				-- useful in checking flow control

			-- Other useful test points available at the component interface:
			-- VBUS_SENSE. Goes high upon cable being plugged in at both ends
			-- USB_RXERROR: USB PHY detects receive errors 
			-- USB_CLK60G: 60 MHz reference clock from the USB PHY through global buffer.
			-- 				Useful in checking input and output signal timing.
			-- USB_RXVALID from PHY (useful in checking the input timing w.r.t. USB_CLK60G.
			);
end component;

component COMSCOPE is 
	port ( 
	--GLOBAL CLOCKS
   	CLK : in std_logic;				-- Master clock for this FPGA
	ASYNC_RESET: in std_logic;		-- Asynchronous reset active high


	-- Choose between signed or unsigned representation
	SIGNED_REPRESENTATION_U_SIGNED_N: in std_logic;
		-- '0'  	Unsigned
		-- '1'  	Signed


	-- Input registers
	REG237: in std_logic_vector(7 downto 0);
	REG238: in std_logic_vector(7 downto 0);
	REG239: in std_logic_vector(7 downto 0);
	REG240: in std_logic_vector(7 downto 0);
	REG241: in std_logic_vector(7 downto 0);
	REG242: in std_logic_vector(7 downto 0);
	REG243: in std_logic_vector(7 downto 0);
	REG244: in std_logic_vector(7 downto 0);
	REG245: in std_logic_vector(7 downto 0);
	REG246: in std_logic_vector(7 downto 0);
	REG247: in std_logic_vector(7 downto 0);
	REG248: in std_logic_vector(7 downto 0);
	REG249: in std_logic_vector(7 downto 0);


	-- Trace 1 input signals
	-- 1-bit signals
	SIGNAL_1_BIT_1_1: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_1_1: in std_logic;
	SIGNAL_1_BIT_1_2: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_1_2: in std_logic;
	SIGNAL_1_BIT_1_3: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_1_3: in std_logic;
	SIGNAL_1_BIT_1_4: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_1_4: in std_logic;

	-- 2-bit signals
	SIGNAL_2_BIT_1_1: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_1_1: in std_logic;
	SIGNAL_2_BIT_1_2: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_1_2: in std_logic;
	SIGNAL_2_BIT_1_3: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_1_3: in std_logic;
	SIGNAL_2_BIT_1_4: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_1_4: in std_logic;

	-- 4-bit signals
	SIGNAL_4_BIT_1_1: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_1_1: in std_logic;
	SIGNAL_4_BIT_1_2: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_1_2: in std_logic;
	SIGNAL_4_BIT_1_3: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_1_3: in std_logic;
	SIGNAL_4_BIT_1_4: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_1_4: in std_logic;

	-- 8-bit signals
	SIGNAL_8_BIT_1_1: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_1_1: in std_logic;
	SIGNAL_8_BIT_1_2: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_1_2: in std_logic;
	SIGNAL_8_BIT_1_3: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_1_3: in std_logic;
	SIGNAL_8_BIT_1_4: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_1_4: in std_logic;

	-- 16-bit signals
	SIGNAL_16_BIT_1_1: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_1_1: in std_logic;
	SIGNAL_16_BIT_1_2: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_1_2: in std_logic;
	SIGNAL_16_BIT_1_3: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_1_3: in std_logic;
	SIGNAL_16_BIT_1_4: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_1_4: in std_logic;
	-- Add signals as necessary (up to 127 signals total per trace)


	-- Trace 2 input signals
	-- 1-bit signals
	SIGNAL_1_BIT_2_1: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_2_1: in std_logic;
	SIGNAL_1_BIT_2_2: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_2_2: in std_logic;
	SIGNAL_1_BIT_2_3: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_2_3: in std_logic;
	SIGNAL_1_BIT_2_4: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_2_4: in std_logic;

	-- 2-bit signals
	SIGNAL_2_BIT_2_1: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_2_1: in std_logic;
	SIGNAL_2_BIT_2_2: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_2_2: in std_logic;
	SIGNAL_2_BIT_2_3: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_2_3: in std_logic;
	SIGNAL_2_BIT_2_4: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_2_4: in std_logic;

	-- 4-bit signals
	SIGNAL_4_BIT_2_1: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_2_1: in std_logic;
	SIGNAL_4_BIT_2_2: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_2_2: in std_logic;
	SIGNAL_4_BIT_2_3: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_2_3: in std_logic;
	SIGNAL_4_BIT_2_4: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_2_4: in std_logic;

	-- 8-bit signals
	SIGNAL_8_BIT_2_1: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_2_1: in std_logic;
	SIGNAL_8_BIT_2_2: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_2_2: in std_logic;
	SIGNAL_8_BIT_2_3: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_2_3: in std_logic;
	SIGNAL_8_BIT_2_4: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_2_4: in std_logic;

	-- 16-bit signals
	SIGNAL_16_BIT_2_1: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_2_1: in std_logic;
	SIGNAL_16_BIT_2_2: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_2_2: in std_logic;
	SIGNAL_16_BIT_2_3: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_2_3: in std_logic;
	SIGNAL_16_BIT_2_4: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_2_4: in std_logic;
	-- Add signals as necessary (up to 127 signals total per trace)


	-- Trace 3 input signals
	-- 1-bit signals
	SIGNAL_1_BIT_3_1: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_3_1: in std_logic;
	SIGNAL_1_BIT_3_2: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_3_2: in std_logic;
	SIGNAL_1_BIT_3_3: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_3_3: in std_logic;
	SIGNAL_1_BIT_3_4: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_3_4: in std_logic;

	-- 2-bit signals
	SIGNAL_2_BIT_3_1: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_3_1: in std_logic;
	SIGNAL_2_BIT_3_2: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_3_2: in std_logic;
	SIGNAL_2_BIT_3_3: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_3_3: in std_logic;
	SIGNAL_2_BIT_3_4: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_3_4: in std_logic;

	-- 4-bit signals
	SIGNAL_4_BIT_3_1: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_3_1: in std_logic;
	SIGNAL_4_BIT_3_2: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_3_2: in std_logic;
	SIGNAL_4_BIT_3_3: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_3_3: in std_logic;
	SIGNAL_4_BIT_3_4: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_3_4: in std_logic;

	-- 8-bit signals
	SIGNAL_8_BIT_3_1: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_3_1: in std_logic;
	SIGNAL_8_BIT_3_2: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_3_2: in std_logic;
	SIGNAL_8_BIT_3_3: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_3_3: in std_logic;
	SIGNAL_8_BIT_3_4: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_3_4: in std_logic;

	-- 16-bit signals
	SIGNAL_16_BIT_3_1: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_3_1: in std_logic;
	SIGNAL_16_BIT_3_2: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_3_2: in std_logic;
	SIGNAL_16_BIT_3_3: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_3_3: in std_logic;
	SIGNAL_16_BIT_3_4: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_3_4: in std_logic;
	-- Add signals as necessary (up to 127 signals total per trace)


	-- Trace 4 input signals
	-- 1-bit signals
	SIGNAL_1_BIT_4_1: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_4_1: in std_logic;
	SIGNAL_1_BIT_4_2: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_4_2: in std_logic;
	SIGNAL_1_BIT_4_3: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_4_3: in std_logic;
	SIGNAL_1_BIT_4_4: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_4_4: in std_logic;

	-- 2-bit signals
	SIGNAL_2_BIT_4_1: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_4_1: in std_logic;
	SIGNAL_2_BIT_4_2: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_4_2: in std_logic;
	SIGNAL_2_BIT_4_3: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_4_3: in std_logic;
	SIGNAL_2_BIT_4_4: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_4_4: in std_logic;

	-- 4-bit signals
	SIGNAL_4_BIT_4_1: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_4_1: in std_logic;
	SIGNAL_4_BIT_4_2: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_4_2: in std_logic;
	SIGNAL_4_BIT_4_3: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_4_3: in std_logic;
	SIGNAL_4_BIT_4_4: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_4_4: in std_logic;

	-- 8-bit signals
	SIGNAL_8_BIT_4_1: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_4_1: in std_logic;
	SIGNAL_8_BIT_4_2: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_4_2: in std_logic;
	SIGNAL_8_BIT_4_3: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_4_3: in std_logic;
	SIGNAL_8_BIT_4_4: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_4_4: in std_logic;

	-- 16-bit signals
	SIGNAL_16_BIT_4_1: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_4_1: in std_logic;
	SIGNAL_16_BIT_4_2: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_4_2: in std_logic;
	SIGNAL_16_BIT_4_3: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_4_3: in std_logic;
	SIGNAL_16_BIT_4_4: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_4_4: in std_logic;
	-- Add signals as necessary (up to 127 signals total per trace)



	-- Trigger signals
	-- 1-bit Triggers
	TRIGGER_1_BIT_1: in std_logic;
	TRIGGER_1_BIT_SAMPLE_CLK_1: in std_logic;
	TRIGGER_1_BIT_2: in std_logic;
	TRIGGER_1_BIT_SAMPLE_CLK_2: in std_logic;
	TRIGGER_1_BIT_3: in std_logic;
	TRIGGER_1_BIT_SAMPLE_CLK_3: in std_logic;
	TRIGGER_1_BIT_4: in std_logic;
	TRIGGER_1_BIT_SAMPLE_CLK_4: in std_logic;

	-- 2-bit Triggers
	TRIGGER_2_BIT_1: in std_logic_vector(1 downto 0);
	TRIGGER_2_BIT_SAMPLE_CLK_1: in std_logic;
	TRIGGER_2_BIT_2: in std_logic_vector(1 downto 0);
	TRIGGER_2_BIT_SAMPLE_CLK_2: in std_logic;
	TRIGGER_2_BIT_3: in std_logic_vector(1 downto 0);
	TRIGGER_2_BIT_SAMPLE_CLK_3: in std_logic;
	TRIGGER_2_BIT_4: in std_logic_vector(1 downto 0);
	TRIGGER_2_BIT_SAMPLE_CLK_4: in std_logic;

	-- 4-bit Triggers
	TRIGGER_4_BIT_1: in std_logic_vector(3 downto 0);
	TRIGGER_4_BIT_SAMPLE_CLK_1: in std_logic;
	TRIGGER_4_BIT_2: in std_logic_vector(3 downto 0);
	TRIGGER_4_BIT_SAMPLE_CLK_2: in std_logic;
	TRIGGER_4_BIT_3: in std_logic_vector(3 downto 0);
	TRIGGER_4_BIT_SAMPLE_CLK_3: in std_logic;
	TRIGGER_4_BIT_4: in std_logic_vector(3 downto 0);
	TRIGGER_4_BIT_SAMPLE_CLK_4: in std_logic;

	-- 8-bit Triggers
	TRIGGER_8_BIT_1: in std_logic_vector(7 downto 0);
	TRIGGER_8_BIT_SAMPLE_CLK_1: in std_logic;
	TRIGGER_8_BIT_2: in std_logic_vector(7 downto 0);
	TRIGGER_8_BIT_SAMPLE_CLK_2: in std_logic;
	TRIGGER_8_BIT_3: in std_logic_vector(7 downto 0);
	TRIGGER_8_BIT_SAMPLE_CLK_3: in std_logic;
	TRIGGER_8_BIT_4: in std_logic_vector(7 downto 0);
	TRIGGER_8_BIT_SAMPLE_CLK_4: in std_logic;

	-- 16-bit Triggers
	TRIGGER_16_BIT_1: in std_logic_vector(15 downto 0);
	TRIGGER_16_BIT_SAMPLE_CLK_1: in std_logic;
	TRIGGER_16_BIT_2: in std_logic_vector(15 downto 0);
	TRIGGER_16_BIT_SAMPLE_CLK_2: in std_logic;
	TRIGGER_16_BIT_3: in std_logic_vector(15 downto 0);
	TRIGGER_16_BIT_SAMPLE_CLK_3: in std_logic;
	TRIGGER_16_BIT_4: in std_logic_vector(15 downto 0);
	TRIGGER_16_BIT_SAMPLE_CLK_4: in std_logic;
	-- Add triggers as necessary (up to 127 trigger signals total)


	-- Output registers
	REG250: out std_logic_vector(7 downto 0);
	REG251: out std_logic_vector(7 downto 0);

	-- Other input signals
	TRIGGER_REARM_TOGGLE: in std_logic;
	FORCE_TRIGGER_TOGGLE: in std_logic;
	REG250_READ: in std_logic;
	START_CAPTURE_TOGGLE: in std_logic
);
end component;

component DAC_TEST is port ( 

	--// Clock
	CLK : in std_logic;		-- Main internal processing clock

	--// User Interface
	ENABLE : in std_logic;		-- test enable

	TPS: out std_logic_vector(10 downto 1);	-- test points
	
	--// DAC interface
	SYNC_B: out std_logic;
	SCLK: out std_logic;	
		-- Maximum frequency of 30 MHz.
		-- doesn't toggle when DAC is inactive
	SDIN: out std_logic

);
end component;

component OUT_TEST is port ( 

	--// Clock
	CLK : in std_logic;		-- Main internal processing clock

	--// User Interface
	ENABLE : in std_logic;		-- test enable

	TPS: out std_logic_vector(10 downto 1);		-- test points

	DATA_OUT: out std_logic_vector(30 downto 1)
);
end component;


component IN_TEST is port ( 
	--// Clock
	CLK : in std_logic;		-- Main internal processing clock

	--// User Interface
	DATA_IN: in std_logic_vector(30 downto 1);
	DIFFERENCE_OUT: out std_logic_vector(30 downto 1);
		-- displays difference between 1 clk delayed data
		-- when not the expected value of 1
	ERROR_CNT: out std_logic_vector(30 downto 1);
		-- counts errors
		-- stable value indicates functional connectors
	ERROR : out std_logic;

	TPS: out std_logic_vector(10 downto 1)		-- test points
);
end component;

COMPONENT com1400_1
PORT(
	CLKIN_IN : IN std_logic;
	RST_IN : IN std_logic;          
	CLKFX_OUT : OUT std_logic;
	CLKIN_IBUFG_OUT : OUT std_logic;
	LOCKED_OUT : OUT std_logic
	);
END COMPONENT;

COMPONENT com1400_2
PORT(
	CLKIN_IN : IN std_logic;
	RST_IN : IN std_logic;          
	CLKFX_OUT : OUT std_logic;
	LOCKED_OUT : OUT std_logic
	);
END COMPONENT;

COMPONENT com1400_3
PORT(
	CLKIN_IN : IN std_logic;
	RST_IN : IN std_logic;          
	CLKFX_OUT : OUT std_logic;
	LOCKED_OUT : OUT std_logic
	);
END COMPONENT;

--------------------------------------------------------
--     ATTRIBUTES
--------------------------------------------------------


-- Pull unused input data pins low in constraint file
--attribute PULLDOWN of LEFT_CONNECTOR<3>: signal is "YES";
-- PULL DOWN INPUT SIGNALS
-- PULL UP INPUT SAMPLING CLOCK AS PER SPECS
--------------------------------------------------------
--     SIGNALS
--------------------------------------------------------
-- Suffix _D indicates a one CLK delayed version of the net with the same name
-- Suffix _E indicates an extended precision version of the net with the same name
-- Suffix _N indicates an inverted version of the net with the same name

--// CONSTANTS
signal ZERO: std_logic;
signal ONE: std_logic;
signal ZERO32: std_logic_vector(31 downto 0);

--// CLOCKS, RESETS
signal CLK_IN_B: std_logic;
signal USB_CLK60G: std_logic;
signal USB_CLK60B: std_logic;
signal CLK_P: std_logic;
signal CLK_IO: std_logic;
signal CLK_ING:	std_logic;
signal DCM_LOCKED_001: std_logic;
signal DCM_LOCKED_002: std_logic;
signal COUNTER8:  std_logic_vector(2 downto 0);
signal ASYNC_RESET_DCM: std_logic;
signal ASYNC_RESET: std_logic;
	-- asynchronous reset
signal RESET_COUNTER: std_logic_vector(17 downto 0) := "000000000000000000";	-- initialize for simulations
	-- spans 2^17/60MHz = 2.18ms

--// INPUT SIGNALS
signal DATA_I_IN: std_logic_vector(9 downto 0);
signal DATA_Q_IN: std_logic_vector(9 downto 0);
signal SAMPLE_CLK_IN: std_logic;
signal DATA_I_IN_A: std_logic_vector(9 downto 0);
signal DATA_Q_IN_A: std_logic_vector(9 downto 0);
signal DATA_I_IN_B: std_logic_vector(9 downto 0);
signal DATA_Q_IN_B: std_logic_vector(9 downto 0);
signal DATA1_I: std_logic_vector(9 downto 0);
signal DATA1_Q: std_logic_vector(9 downto 0);
signal A_BN: std_logic;
signal A_BN_DF: std_logic;
signal DATA1A_I: std_logic_vector(11 downto 0);
signal DATA1A_Q: std_logic_vector(11 downto 0);

--// USB Interface for M&C
signal RX_USB: std_logic_vector(7 downto 0); 
signal RX_USB_SAMPLE_CLK_REQ: std_logic;
signal RX_USB_BUFFER_EMPTY: std_logic;
signal RX_USB_BUFFER_EMPTY_D: std_logic;
signal RX_USB_BUFFER_EMPTY_D2: std_logic;
signal TX_USB: std_logic_vector(7 downto 0); 
signal TX_USB_SAMPLE_CLK: std_logic;
signal TX_USB_TOGGLE: std_logic;
signal TX_USB_TOGGLE_D: std_logic;
signal TX_USB_TOGGLE_D2: std_logic;
signal USB_TP: std_logic_vector(10 downto 1);

--// ComScope traces
signal S1_1: std_logic_vector(7 downto 0);	
signal S1_1_CLK: std_logic;
signal S1_1B_CLK: std_logic;
signal S1_2: std_logic_vector(7 downto 0);	
signal S1_2_CLK: std_logic;
signal S1_2B_CLK: std_logic;
signal S1_3: std_logic_vector(7 downto 0);	
signal S1_3_CLK: std_logic;
signal S1_3B_CLK: std_logic;
signal S1_4: std_logic_vector(7 downto 0);	
signal S1_4_CLK: std_logic;
signal S1_4B_CLK: std_logic;

signal S2_1: std_logic_vector(7 downto 0);	
signal S2_1_CLK: std_logic;
signal S2_1B_CLK: std_logic;
signal S2_2: std_logic_vector(7 downto 0);	
signal S2_2_CLK: std_logic;
signal S2_2B_CLK: std_logic;
signal S2_3: std_logic_vector(7 downto 0);	
signal S2_3_CLK: std_logic;
signal S2_3B_CLK: std_logic;
signal S2_4: std_logic_vector(7 downto 0);	
signal S2_4_CLK: std_logic;
signal S2_4B_CLK: std_logic;

signal T1: std_logic;
signal T1_CLK: std_logic;
signal T2: std_logic;
signal T2_CLK: std_logic;

--// TEST POINTS
signal TP1: std_logic_vector(10 downto 1);
signal TP2: std_logic_vector(10 downto 1);

signal TRIGGER_REARM_TOGGLE: std_logic:= '0';
signal FORCE_TRIGGER_TOGGLE: std_logic:= '0';
signal START_CAPTURE_TOGGLE: std_logic:= '0';
signal REG250_READ: std_logic;

--// Atmel uc interface
signal UC_ADDRESS: std_logic_vector(7 downto 0);
signal UC_ADDR_D: std_logic_vector(7 downto 0);
signal UC_ADDR_D2: std_logic_vector(7 downto 0);
signal UC_RDN_D: std_logic;
signal UC_RDN_D2: std_logic;
signal CONFIG_CHANGE_TOGGLE: std_logic;
signal CONFIG_CHANGE_TOGGLE_D: std_logic;
signal CONFIG_CHANGE_TOGGLE_D2: std_logic;
signal CONFIG_CHANGE_PULSE: std_logic;
signal UC_ALE: std_logic;
signal UC_ALE_B: std_logic;
signal UC_WRN: std_logic;
signal UC_WRN_B: std_logic;

--// Control Registers
signal REG0: std_logic_vector(7 downto 0);
signal REG1: std_logic_vector(7 downto 0);
signal REG2: std_logic_vector(7 downto 0);
signal REG3: std_logic_vector(7 downto 0);
signal REG4: std_logic_vector(7 downto 0);
signal REG5: std_logic_vector(7 downto 0);
signal REG6: std_logic_vector(7 downto 0);
signal REG7: std_logic_vector(7 downto 0);
signal REG8: std_logic_vector(7 downto 0);
signal REG9: std_logic_vector(7 downto 0);
signal REG255: std_logic_vector(7 downto 0);

--// Status Registers
signal SREG0: std_logic_vector(7 downto 0);
signal SREG1: std_logic_vector(7 downto 0);
signal SREG2: std_logic_vector(7 downto 0);
signal SREG3: std_logic_vector(7 downto 0);
signal SREG4: std_logic_vector(7 downto 0);
signal SREG5: std_logic_vector(7 downto 0);
signal SREG6: std_logic_vector(7 downto 0);
signal SREG7: std_logic_vector(7 downto 0);
signal SREG8: std_logic_vector(7 downto 0);
signal SREG9: std_logic_vector(7 downto 0);

--// ComScope control registers
signal REG237: std_logic_vector(7 downto 0):= "00000000";
signal REG238: std_logic_vector(7 downto 0):= "00000000";
signal REG239: std_logic_vector(7 downto 0):= "00000000";
signal REG240: std_logic_vector(7 downto 0):= "00001101";	-- Trace 1 signal selection
signal REG241: std_logic_vector(7 downto 0):= "00000000";	-- Trigger position, decimation
signal REG242: std_logic_vector(7 downto 0):= "00001110";	-- Trace 2 signal selection
signal REG243: std_logic_vector(7 downto 0):= "00000000";	-- Trigger position, decimation
signal REG244: std_logic_vector(7 downto 0):= "00000000";	-- Trace 3 signal selection
signal REG245: std_logic_vector(7 downto 0):= "00000000";	-- Trigger position, decimation
signal REG246: std_logic_vector(7 downto 0):= "00000000";	-- Trace 4 signal selection
signal REG247: std_logic_vector(7 downto 0):= "00000000";	-- Trigger position, decimation
signal REG248: std_logic_vector(7 downto 0):= "10001101";	-- Trigger edge, trigger signal
signal REG249: std_logic_vector(7 downto 0):= "10111111";	-- Trigger threshold
signal REG250: std_logic_vector(7 downto 0):= "00000000";
signal REG251: std_logic_vector(7 downto 0):= "00000000";

signal DIFFERENCE_OUT: std_logic_vector(30 downto 1);
signal ERROR_CNT: std_logic_vector(30 downto 1);
signal TEST_DATA_OUT: std_logic_vector(30 downto 1);
signal TEST_DATA_IN: std_logic_vector(30 downto 1);

--// FPGA option and revision (ASCII format)
-- These two ASCII characters appear in the Comblock Control Center.
-- '0' = 48
-- 'A' = 65
constant OPTION: std_logic_vector(7 downto 0) := x"20";	-- '-'
constant REVISION: std_logic_vector(7 downto 0) := x"36";	-- '4'
--------------------------------------------------------
--      IMPLEMENTATION
--------------------------------------------------------

begin
--// CONSTANTS ---------------------
ZERO <= '0';
ONE <= '1';
ZERO32 <= (others => '0');


--// CLOCKS, RESET ---------------------
-- Generate two types of asynchronous resets:
-- one (ASYNC_RESET_DCM) to hold the DCMs in reset for about 1ms after configuration
-- while the USB PHY switches from 30 MHz clock to 60 MHz clock.
-- The other (ASYNC_RESET) to wait until the DCM stabilize to their
-- desired frequencies. Takes about 100 uS.

RESET_COUNTER_GEN_001: process(USB_CLK60G)
begin
	if rising_edge(USB_CLK60G) then
		ASYNC_RESET <= not(RESET_COUNTER(17));	-- about 2 ms.

		if(RESET_COUNTER(17) = '0') then
			RESET_COUNTER <= RESET_COUNTER + 1;
	  	end if;

		-- Reset is synchronous with USB_CLK60G but asynchronous with CLK_P and CLK_IO.
		-- reclocked to prevent glitches (would reset DCM).
		if (RESET_COUNTER(17 downto 16) = "00") then
			ASYNC_RESET_DCM <= '1';	-- about 1ms
		else
			ASYNC_RESET_DCM <= '0';	-- about 1ms
		end if;			
  	end if;
end process;

-- force selection of correct IBUF
IBUF_001: IBUF port map(I => UC_WRN_IN, O => UC_WRN_B);  
IBUF_002: IBUF port map(I => UC_ALE_IN, O => UC_ALE_B); 
IBUF_003: IBUF port map(I => LEFT_CONNECTOR(1), O => CLK_IN_B);



-- Global buffers
BUFG_001: BUFG  port map (I=> USB_CLK60B, O=> USB_CLK60G);	
	-- reference clock from USB PHY. 
BUFG_004: BUFG  port map (I=> CLK_IN_B, O=> CLK_ING);	
	-- external input clock
BUFG_006: BUFG port map(I => UC_WRN_B, O => UC_WRN);  
BUFG_007: BUFG port map(I => UC_ALE_B, O => UC_ALE); 


-- DCM used as frequency synthesizer for processing clock. 60 MHz x M / D = 90 MHz,
--  where M = 3, D = 2.  M and D are set through the attributes above.
Inst_com1400_1: com1400_1 PORT MAP(
	CLKIN_IN => USB_CLK60,		    	-- input pin
	RST_IN => ASYNC_RESET_DCM,
	CLKFX_OUT => CLK_P,	    			-- after BUFG
	CLKIN_IBUFG_OUT => USB_CLK60B,	-- after IBUFG
	LOCKED_OUT => DCM_LOCKED_001
);



-- DCM used as frequency synthesizer for I/O clock. 60 MHz x M / D = 40 MHz, 
-- where M = 2, D = 3. M and D are set through the attributes above.
Inst_com1400_2: com1400_2 PORT MAP(
	CLKIN_IN => USB_CLK60G,
	RST_IN => ASYNC_RESET_DCM,
	CLKFX_OUT => CLK_IO,
	LOCKED_OUT => DCM_LOCKED_002
);

IN_TEST_001: IN_TEST PORT MAP(
	CLK => CLK_ING,
	--// User Interface
	ERROR => TP1(1),
	TPS(1) => TP1(2),
	TPS(2) => TP1(3),
	TPS(10 downto 3) => open,
	DIFFERENCE_OUT => DIFFERENCE_OUT, 
	ERROR_CNT => ERROR_CNT,
	DATA_IN => TEST_DATA_IN
);

TEST_DATA_IN(8 downto 1) <= LEFT_CONNECTOR(9 downto 2);
TEST_DATA_IN(17 downto 9) <=  LEFT_CONNECTOR(19 downto 11);
TEST_DATA_IN(26 downto 18) <=  LEFT_CONNECTOR(29 downto 21);
TEST_DATA_IN(30 downto 27) <= LEFT_CONNECTOR(34 downto 31);

OUT_TEST_001: OUT_TEST PORT MAP(
	CLK => CLK_P,
	--// User Interface
	ENABLE => ONE,
	TPS(1) => TP2(1),
	TPS(10 downto 2) => open,
	DATA_OUT => TEST_DATA_OUT
);

BOTTOM_CONNECTOR(9 downto 2) <= TEST_DATA_OUT(8 downto 1);
BOTTOM_CONNECTOR(19 downto 11) <= TEST_DATA_OUT(17 downto 9);
BOTTOM_CONNECTOR(29 downto 21) <= TEST_DATA_OUT(26 downto 18);
BOTTOM_CONNECTOR(34 downto 31) <= TEST_DATA_OUT(30 downto 27);

RIGHT_CONNECTOR(9 downto 2) <= TEST_DATA_OUT(8 downto 1);
RIGHT_CONNECTOR(19 downto 11) <= TEST_DATA_OUT(17 downto 9);
RIGHT_CONNECTOR(29 downto 21) <= TEST_DATA_OUT(26 downto 18);
RIGHT_CONNECTOR(34 downto 31)	<= TEST_DATA_OUT(30 downto 27);

-- Test clocks. Generate a divide-by-8 clock and forward to 
-- the INIT# test point (located next to the 'DONE' test point)
-- This is an easy way to verify that the VHDL code is properly implemented
-- and that no glitches exist on an external or internal clock.
process(CLK_P)
begin
	if rising_edge(CLK_P) then
		COUNTER8 <= COUNTER8 + 1;
	end if;
end process;
INITB <= COUNTER8(2);

--// INPUT SIGNALS
-- Convert from external input clock to local processing clock areas via dual-port RAM

-- Immediately reclock all inputs before processing. Use the external clock
RECLOCK_INPUT_001: process(CLK_ING, LEFT_CONNECTOR)
begin
	if rising_edge(CLK_ING) then
		if(LEFT_CONNECTOR(2) = '1') then
			DATA_I_IN(9) <= LEFT_CONNECTOR(3);
			DATA_I_IN(8) <= LEFT_CONNECTOR(4);
			DATA_I_IN(7) <= LEFT_CONNECTOR(5);
			DATA_I_IN(6) <= LEFT_CONNECTOR(6);
			DATA_I_IN(5) <= LEFT_CONNECTOR(7);
			DATA_I_IN(4) <= LEFT_CONNECTOR(8);
			DATA_I_IN(3) <= LEFT_CONNECTOR(9);
			DATA_I_IN(2) <= LEFT_CONNECTOR(11);
			DATA_I_IN(1) <= LEFT_CONNECTOR(12);
			DATA_I_IN(0) <= LEFT_CONNECTOR(13);

			DATA_Q_IN(9) <= LEFT_CONNECTOR(14);
			DATA_Q_IN(8) <= LEFT_CONNECTOR(15);
			DATA_Q_IN(7) <= LEFT_CONNECTOR(16);
			DATA_Q_IN(6) <= LEFT_CONNECTOR(17);
			DATA_Q_IN(5) <= LEFT_CONNECTOR(18);
			DATA_Q_IN(4) <= LEFT_CONNECTOR(19);
			DATA_Q_IN(3) <= LEFT_CONNECTOR(21);
			DATA_Q_IN(2) <= LEFT_CONNECTOR(22);
			DATA_Q_IN(1) <= LEFT_CONNECTOR(23);
			DATA_Q_IN(0) <= LEFT_CONNECTOR(24);
		end if;
	end if;
end process;

-- LEFT_CONNECTOR(3 to 24) are input pins only
--LEFT_CONNECTOR(9 downto 2) <= (others => 'Z');
--LEFT_CONNECTOR(19 downto 11) <= (others => 'Z');
--LEFT_CONNECTOR(24 downto 21) <= (others => 'Z');

-- write to A side or B side
RECLOCK_INPUT_002: process(CLK_ING)
begin
	if rising_edge(CLK_ING) then
		A_BN <= not A_BN;
		if(A_BN = '0') then
			-- direct to A side
			DATA_I_IN_A <= DATA_I_IN;
			DATA_Q_IN_A <= DATA_Q_IN;
		else
			-- direct to B side
			DATA_I_IN_B <= DATA_I_IN;
			DATA_Q_IN_B <= DATA_Q_IN;
		end if;
	end if;
end process;

-- reclock A_BN flag at the falling edge of CLK_ING
RECLOCK_INPUT_003: process(CLK_ING)
begin
	if falling_edge(CLK_ING) then
		A_BN_DF <= A_BN;
	end if;
end process;

-- reclock input samples with processing clock when input samples are 
-- stable in the A or B buffer.
RECLOCK_INPUT_004: process(CLK_P, A_BN_DF, DATA_I_IN_A, DATA_I_IN_B, 
	DATA_Q_IN_A, DATA_Q_IN_B)
begin
	if rising_edge(CLK_P) then
		if(A_BN_DF = '1') then
			-- read from stable A side
			DATA1_I <= DATA_I_IN_A;
			DATA1_Q <= DATA_Q_IN_A;
		else
			-- read from stable B side
			DATA1_I <= DATA_I_IN_B;
			DATA1_Q <= DATA_Q_IN_B;
		end if;
	end if;
end process;

--// INSERT YOUR CODE HERE




--// STATUS MONITORING REGISTERS
SREG0 <= DIFFERENCE_OUT(8 downto 1);
SREG1 <= DIFFERENCE_OUT(16 downto 9);
SREG2 <= DIFFERENCE_OUT(24 downto 17);
SREG3(5 downto 0) <= DIFFERENCE_OUT(30 downto 25);
SREG4 <= ERROR_CNT(8 downto 1);
SREG5 <= ERROR_CNT(16 downto 9);
SREG6 <= ERROR_CNT(24 downto 17);
SREG7(5 downto 0) <= ERROR_CNT(30 downto 25);
--SREG4 <= ;
--SREG5 <= ;
--SREG6 <= ;
--SREG7 <= ;
--SREG8 <= ;
--SREG9 <= ;

--// USE YOUR CONTROL REGISTERS HERE
-- <= REG0;
-- <= REG1;
-- <= REG2;
-- <= REG3;
-- <= REG4;
-- <= REG5;
-- <= REG6;
-- <= REG7;
-- <= REG8;
-- <= REG9;


--// USB interface
-- Channel 1 for M&C
-- Channel 2 for demodulated data rx.
USB20_001: USB20 port map(
	USB_CLK60G => USB_CLK60G,
	ASYNC_RESET => ASYNC_RESET,
	CLK_P => CLK_P,

	-- interface with USB PHY. Direct connection to I/O pins.
	USB_RESET => USB_RESET,
--	USB_DATABUS16_8 => USB_DATABUS16_8,
--	USB_SUSPENDN => USB_SUSPENDN,
	USB_XCVRSELECT => USB_XCVRSELECT,
	USB_TERMSELECT => USB_TERMSELECT,
	USB_OPMODE => USB_OPMODE,
	USB_LINESTATE => USB_LINESTATE,
	USB_TXVALID => USB_TXVALID,
	USB_TXREADY => USB_TXREADY,
	USB_RXVALID => USB_RXVALID,
	USB_RXACTIVE => USB_RXACTIVE,
	USB_RXERROR => USB_RXERROR,
	USB_DATA_IN => USB_DATA_IN,
	USB_DATA_OUT => USB_DATA_OUT,
	USB_VBUS_SENSE => USB_VBUS_SENSE,

	-- M&C channel. Interfaces with Atmel uC. Uses data stream 1
	DATA1_OUT => RX_USB,
	DATA1_OUT_SAMPLE_CLK => open,
	DATA1_OUT_BUFFER_EMPTY => RX_USB_BUFFER_EMPTY,
	DATA1_OUT_SAMPLE_CLK_REQ => RX_USB_SAMPLE_CLK_REQ,
	DATA1_IN => TX_USB,
   DATA1_IN_SAMPLE_CLK => TX_USB_SAMPLE_CLK,
	DATA1_IN_SAMPLE_CLK_REQ => open,	
		-- Atmel uC is much slower than USB. no need to verify tx flow control.

	-- Payload channel. Data stream 2
	DATA2_OUT => open,
	DATA2_OUT_SAMPLE_CLK => open,
	DATA2_OUT_BUFFER_EMPTY => open,
	DATA2_OUT_SAMPLE_CLK_REQ => ZERO,
	DATA2_IN => ZERO32(7 downto 0),
	DATA2_IN_SAMPLE_CLK => ZERO,
	DATA2_IN_SAMPLE_CLK_REQ => open,

	USB_TP => USB_TP
);


DAC_TEST_001: DAC_TEST port map(		
	CLK => CLK_P,
	--// User Interface
	ENABLE => ONE,
	TPS => open,
  	--// DAC interface
--  	SYNC_B => TP1(1),
--  	SCLK => TP1(2),
--  	SDIN => TP1(3)
  	SYNC_B => SYNC_B,
  	SCLK => SCLK,
  	SDIN => SDIN
);

--  	SYNC_B <= TP1(1);
--  	SCLK <= TP1(2);
--  	SDIN <= TP1(3);

--// RIGHT OUTPUT CONNECTOR
--// DEFINE YOUR OUTPUT PINS HERE
--RIGHT_CONNECTOR(1) <= not CLK_IO;	
RIGHT_CONNECTOR(1) <= CLK_P;	

--// BOTTOM OUTPUT CONNECTOR
--// DEFINE YOUR OUTPUT PINS HERE
--BOTTOM_CONNECTOR(1) <= not CLK_IO;	
BOTTOM_CONNECTOR(1) <= CLK_P;	

		
--// Comscope --------------
-- Define Comscope traces here 
-- Trace 1
--S1_1 <= SDRAM_A_LOCAL(7 downto 0);
--S1_1_CLK <= '1';
--S1_2 <= "000" & SDRAM_A_LOCAL(12 downto 8);
--S1_2_CLK <= '1';
--S1_3 <= SDRAM_DQ_LOCAL(7 downto 0);
--S1_3_CLK <= '1';
--S1_4 <= SDRAM_DQ_LOCAL(15 downto 8);
--S1_4_CLK <= '1';

-- Trace 2
--S2_1 <= SDRAM_BA0_LOCAL & SDRAM_BA1_LOCAL & SDRAM_CAS_N_LOCAL & SDRAM_RAS_N_LOCAL 
--	& SDRAM_WE_N_LOCAL & SDRAM_CKE_LOCAL & SDRAM_DQML_LOCAL & SDRAM_DQMH_LOCAL;
--S2_1_CLK <= '1';

-- Trigger (1-bit format, undefined)
--T1_CLK <= '1';
--T2 <= CONFIG_CHANGE_PULSE;	-- START TEST

S1_1B_CLK <= S1_1_CLK or REG240(7);	-- Nominal sampling clock or real-time processing clock
S1_2B_CLK <= S1_2_CLK or REG240(7);	-- Nominal sampling clock or real-time processing clock
S1_3B_CLK <= S1_3_CLK or REG240(7);	-- Nominal sampling clock or real-time processing clock
S1_4B_CLK <= S1_4_CLK or REG240(7);	-- Nominal sampling clock or real-time processing clock

S2_1B_CLK <= S2_1_CLK or REG242(7);	-- Nominal sampling clock or real-time processing clock
S2_2B_CLK <= S2_2_CLK or REG242(7);	-- Nominal sampling clock or real-time processing clock
S2_3B_CLK <= S2_3_CLK or REG242(7);	-- Nominal sampling clock or real-time processing clock
S2_4B_CLK <= S2_4_CLK or REG242(7);	-- Nominal sampling clock or real-time processing clock

COMSCOPE_001: COMSCOPE port map(
   CLK => CLK_P,
	ASYNC_RESET => ASYNC_RESET,
	SIGNED_REPRESENTATION_U_SIGNED_N => REG248(6),	-- trigger format signed(1) or unsigned(0)
	REG237 => REG237,
	REG238 => REG238,
	REG239 => REG239,
	REG240 => REG240,
	REG241 => REG241,
	REG242 => REG242,
	REG243 => REG243,
	REG244 => REG244,
	REG245 => REG245,
	REG246 => REG246,
	REG247 => REG247,
	REG248 => REG248,
	REG249 => REG249,
	SIGNAL_1_BIT_1_1 => ZERO,
	SIGNAL_1_BIT_SAMPLE_CLK_1_1 => ZERO,
	SIGNAL_1_BIT_1_2 => ZERO,
	SIGNAL_1_BIT_SAMPLE_CLK_1_2 => ZERO,
	SIGNAL_1_BIT_1_3 => ZERO,
	SIGNAL_1_BIT_SAMPLE_CLK_1_3 => ZERO,
	SIGNAL_1_BIT_1_4 => ZERO,
	SIGNAL_1_BIT_SAMPLE_CLK_1_4 => ZERO,
	SIGNAL_2_BIT_1_1 => ZERO32(1 downto 0),
	SIGNAL_2_BIT_SAMPLE_CLK_1_1 => ZERO,
	SIGNAL_2_BIT_1_2 => ZERO32(1 downto 0),
	SIGNAL_2_BIT_SAMPLE_CLK_1_2 => ZERO,
	SIGNAL_2_BIT_1_3 => ZERO32(1 downto 0),
	SIGNAL_2_BIT_SAMPLE_CLK_1_3 => ZERO,
	SIGNAL_2_BIT_1_4 => ZERO32(1 downto 0),
	SIGNAL_2_BIT_SAMPLE_CLK_1_4 => ZERO,
	SIGNAL_4_BIT_1_1 => ZERO32(3 downto 0),
	SIGNAL_4_BIT_SAMPLE_CLK_1_1 => ZERO,
	SIGNAL_4_BIT_1_2 => ZERO32(3 downto 0),
	SIGNAL_4_BIT_SAMPLE_CLK_1_2 => ZERO,
	SIGNAL_4_BIT_1_3 => ZERO32(3 downto 0),
	SIGNAL_4_BIT_SAMPLE_CLK_1_3 => ZERO,
	SIGNAL_4_BIT_1_4 => ZERO32(3 downto 0),
	SIGNAL_4_BIT_SAMPLE_CLK_1_4 => ZERO,
	SIGNAL_8_BIT_1_1 => S1_1,
	SIGNAL_8_BIT_SAMPLE_CLK_1_1 => S1_1B_CLK,
	SIGNAL_8_BIT_1_2 => S1_2,
	SIGNAL_8_BIT_SAMPLE_CLK_1_2 => S1_2B_CLK,
	SIGNAL_8_BIT_1_3 => S1_3,
	SIGNAL_8_BIT_SAMPLE_CLK_1_3 => S1_3B_CLK,
	SIGNAL_8_BIT_1_4 => S1_4,
	SIGNAL_8_BIT_SAMPLE_CLK_1_4 => S1_4B_CLK,
	SIGNAL_16_BIT_1_1 => ZERO32(15 downto 0),
	SIGNAL_16_BIT_SAMPLE_CLK_1_1 => ZERO,
	SIGNAL_16_BIT_1_2 => ZERO32(15 downto 0),
	SIGNAL_16_BIT_SAMPLE_CLK_1_2 => ZERO,
	SIGNAL_16_BIT_1_3 => ZERO32(15 downto 0),
	SIGNAL_16_BIT_SAMPLE_CLK_1_3 => ZERO,
	SIGNAL_16_BIT_1_4 => ZERO32(15 downto 0),
	SIGNAL_16_BIT_SAMPLE_CLK_1_4 => ZERO,
	SIGNAL_1_BIT_2_1 => ZERO,
	SIGNAL_1_BIT_SAMPLE_CLK_2_1 => ZERO,
	SIGNAL_1_BIT_2_2 => ZERO,
	SIGNAL_1_BIT_SAMPLE_CLK_2_2 => ZERO,
	SIGNAL_1_BIT_2_3 => ZERO,
	SIGNAL_1_BIT_SAMPLE_CLK_2_3 => ZERO,
	SIGNAL_1_BIT_2_4 => ZERO,
	SIGNAL_1_BIT_SAMPLE_CLK_2_4 => ZERO,
	SIGNAL_2_BIT_2_1 => ZERO32(1 downto 0),
	SIGNAL_2_BIT_SAMPLE_CLK_2_1 => ZERO,
	SIGNAL_2_BIT_2_2 => ZERO32(1 downto 0),
	SIGNAL_2_BIT_SAMPLE_CLK_2_2 => ZERO,
	SIGNAL_2_BIT_2_3 => ZERO32(1 downto 0),
	SIGNAL_2_BIT_SAMPLE_CLK_2_3 => ZERO,
	SIGNAL_2_BIT_2_4 => ZERO32(1 downto 0),
	SIGNAL_2_BIT_SAMPLE_CLK_2_4 => ZERO,
	SIGNAL_4_BIT_2_1 => ZERO32(3 downto 0),
	SIGNAL_4_BIT_SAMPLE_CLK_2_1 => ZERO,
	SIGNAL_4_BIT_2_2 => ZERO32(3 downto 0),
	SIGNAL_4_BIT_SAMPLE_CLK_2_2 => ZERO,
	SIGNAL_4_BIT_2_3 => ZERO32(3 downto 0),
	SIGNAL_4_BIT_SAMPLE_CLK_2_3 => ZERO,
	SIGNAL_4_BIT_2_4 => ZERO32(3 downto 0),
	SIGNAL_4_BIT_SAMPLE_CLK_2_4 => ZERO,
	SIGNAL_8_BIT_2_1 => S2_1,
	SIGNAL_8_BIT_SAMPLE_CLK_2_1 => S2_1B_CLK,
	SIGNAL_8_BIT_2_2 => S2_2,
	SIGNAL_8_BIT_SAMPLE_CLK_2_2 => S2_2B_CLK,
	SIGNAL_8_BIT_2_3 => S2_3,
	SIGNAL_8_BIT_SAMPLE_CLK_2_3 => S2_3B_CLK,
	SIGNAL_8_BIT_2_4 => S2_4,
	SIGNAL_8_BIT_SAMPLE_CLK_2_4 => S2_4B_CLK,
	SIGNAL_16_BIT_2_1 => ZERO32(15 downto 0),
	SIGNAL_16_BIT_SAMPLE_CLK_2_1 => ZERO,
	SIGNAL_16_BIT_2_2 => ZERO32(15 downto 0),
	SIGNAL_16_BIT_SAMPLE_CLK_2_2 => ZERO,
	SIGNAL_16_BIT_2_3 => ZERO32(15 downto 0),
	SIGNAL_16_BIT_SAMPLE_CLK_2_3 => ZERO,
	SIGNAL_16_BIT_2_4 => ZERO32(15 downto 0),
	SIGNAL_16_BIT_SAMPLE_CLK_2_4 => ZERO,
	SIGNAL_1_BIT_3_1 => ZERO,
	SIGNAL_1_BIT_SAMPLE_CLK_3_1 => ZERO,
	SIGNAL_1_BIT_3_2 => ZERO,
	SIGNAL_1_BIT_SAMPLE_CLK_3_2 => ZERO,
	SIGNAL_1_BIT_3_3 => ZERO,
	SIGNAL_1_BIT_SAMPLE_CLK_3_3 => ZERO,
	SIGNAL_1_BIT_3_4 => ZERO,
	SIGNAL_1_BIT_SAMPLE_CLK_3_4 => ZERO,
	SIGNAL_2_BIT_3_1 => ZERO32(1 downto 0),
	SIGNAL_2_BIT_SAMPLE_CLK_3_1 => ZERO,
	SIGNAL_2_BIT_3_2 => ZERO32(1 downto 0),
	SIGNAL_2_BIT_SAMPLE_CLK_3_2 => ZERO,
	SIGNAL_2_BIT_3_3 => ZERO32(1 downto 0),
	SIGNAL_2_BIT_SAMPLE_CLK_3_3 => ZERO,
	SIGNAL_2_BIT_3_4 => ZERO32(1 downto 0),
	SIGNAL_2_BIT_SAMPLE_CLK_3_4 => ZERO,
	SIGNAL_4_BIT_3_1 => ZERO32(3 downto 0),
	SIGNAL_4_BIT_SAMPLE_CLK_3_1 => ZERO,
	SIGNAL_4_BIT_3_2 => ZERO32(3 downto 0),
	SIGNAL_4_BIT_SAMPLE_CLK_3_2 => ZERO,
	SIGNAL_4_BIT_3_3 => ZERO32(3 downto 0),
	SIGNAL_4_BIT_SAMPLE_CLK_3_3 => ZERO,
	SIGNAL_4_BIT_3_4 => ZERO32(3 downto 0),
	SIGNAL_4_BIT_SAMPLE_CLK_3_4 => ZERO,
	SIGNAL_8_BIT_3_1 => ZERO32(7 downto 0),
	SIGNAL_8_BIT_SAMPLE_CLK_3_1 => ZERO,
	SIGNAL_8_BIT_3_2 => ZERO32(7 downto 0),
	SIGNAL_8_BIT_SAMPLE_CLK_3_2 => ZERO,
	SIGNAL_8_BIT_3_3 => ZERO32(7 downto 0),
	SIGNAL_8_BIT_SAMPLE_CLK_3_3 => ZERO,
	SIGNAL_8_BIT_3_4 => ZERO32(7 downto 0),
	SIGNAL_8_BIT_SAMPLE_CLK_3_4 => ZERO,
	SIGNAL_16_BIT_3_1 => ZERO32(15 downto 0),
	SIGNAL_16_BIT_SAMPLE_CLK_3_1 => ZERO,
	SIGNAL_16_BIT_3_2 => ZERO32(15 downto 0),
	SIGNAL_16_BIT_SAMPLE_CLK_3_2 => ZERO,
	SIGNAL_16_BIT_3_3 => ZERO32(15 downto 0),
	SIGNAL_16_BIT_SAMPLE_CLK_3_3 => ZERO,
	SIGNAL_16_BIT_3_4 => ZERO32(15 downto 0),
	SIGNAL_16_BIT_SAMPLE_CLK_3_4 => ZERO,
	SIGNAL_1_BIT_4_1 => ZERO,
	SIGNAL_1_BIT_SAMPLE_CLK_4_1 => ZERO,
	SIGNAL_1_BIT_4_2 => ZERO,
	SIGNAL_1_BIT_SAMPLE_CLK_4_2 => ZERO,
	SIGNAL_1_BIT_4_3 => ZERO,
	SIGNAL_1_BIT_SAMPLE_CLK_4_3 => ZERO,
	SIGNAL_1_BIT_4_4 => ZERO,
	SIGNAL_1_BIT_SAMPLE_CLK_4_4 => ZERO,
	SIGNAL_2_BIT_4_1 => ZERO32(1 downto 0),
	SIGNAL_2_BIT_SAMPLE_CLK_4_1 => ZERO,
	SIGNAL_2_BIT_4_2 => ZERO32(1 downto 0),
	SIGNAL_2_BIT_SAMPLE_CLK_4_2 => ZERO,
	SIGNAL_2_BIT_4_3 => ZERO32(1 downto 0),
	SIGNAL_2_BIT_SAMPLE_CLK_4_3 => ZERO,
	SIGNAL_2_BIT_4_4 => ZERO32(1 downto 0),
	SIGNAL_2_BIT_SAMPLE_CLK_4_4 => ZERO,
	SIGNAL_4_BIT_4_1 => ZERO32(3 downto 0),
	SIGNAL_4_BIT_SAMPLE_CLK_4_1 => ZERO,
	SIGNAL_4_BIT_4_2 => ZERO32(3 downto 0),
	SIGNAL_4_BIT_SAMPLE_CLK_4_2 => ZERO,
	SIGNAL_4_BIT_4_3 => ZERO32(3 downto 0),
	SIGNAL_4_BIT_SAMPLE_CLK_4_3 => ZERO,
	SIGNAL_4_BIT_4_4 => ZERO32(3 downto 0),
	SIGNAL_4_BIT_SAMPLE_CLK_4_4 => ZERO,
	SIGNAL_8_BIT_4_1 => ZERO32(7 downto 0),
	SIGNAL_8_BIT_SAMPLE_CLK_4_1 => ZERO,
	SIGNAL_8_BIT_4_2 => ZERO32(7 downto 0),
	SIGNAL_8_BIT_SAMPLE_CLK_4_2 => ZERO,
	SIGNAL_8_BIT_4_3 => ZERO32(7 downto 0),
	SIGNAL_8_BIT_SAMPLE_CLK_4_3 => ZERO,
	SIGNAL_8_BIT_4_4 => ZERO32(7 downto 0),
	SIGNAL_8_BIT_SAMPLE_CLK_4_4 => ZERO,
	SIGNAL_16_BIT_4_1 => ZERO32(15 downto 0),
	SIGNAL_16_BIT_SAMPLE_CLK_4_1 => ZERO,
	SIGNAL_16_BIT_4_2 => ZERO32(15 downto 0),
	SIGNAL_16_BIT_SAMPLE_CLK_4_2 => ZERO,
	SIGNAL_16_BIT_4_3 => ZERO32(15 downto 0),
	SIGNAL_16_BIT_SAMPLE_CLK_4_3 => ZERO,
	SIGNAL_16_BIT_4_4 => ZERO32(15 downto 0),
	SIGNAL_16_BIT_SAMPLE_CLK_4_4 => ZERO,
	TRIGGER_1_BIT_1 => T1,
	TRIGGER_1_BIT_SAMPLE_CLK_1 => T1_CLK,
	TRIGGER_1_BIT_2 => T2,
	TRIGGER_1_BIT_SAMPLE_CLK_2 => T2_CLK,
	TRIGGER_1_BIT_3 => ZERO,
	TRIGGER_1_BIT_SAMPLE_CLK_3 => ZERO,
	TRIGGER_1_BIT_4 => ZERO,
	TRIGGER_1_BIT_SAMPLE_CLK_4 => ZERO,
	TRIGGER_2_BIT_1 => ZERO32(1 downto 0),
	TRIGGER_2_BIT_SAMPLE_CLK_1 => ZERO,
	TRIGGER_2_BIT_2 => ZERO32(1 downto 0),
	TRIGGER_2_BIT_SAMPLE_CLK_2 => ZERO,
	TRIGGER_2_BIT_3 => ZERO32(1 downto 0),
	TRIGGER_2_BIT_SAMPLE_CLK_3 => ZERO,
	TRIGGER_2_BIT_4 => ZERO32(1 downto 0),
	TRIGGER_2_BIT_SAMPLE_CLK_4 => ZERO,
	TRIGGER_4_BIT_1 => ZERO32(3 downto 0),
	TRIGGER_4_BIT_SAMPLE_CLK_1 => ZERO,
	TRIGGER_4_BIT_2 => ZERO32(3 downto 0),
	TRIGGER_4_BIT_SAMPLE_CLK_2 => ZERO,
	TRIGGER_4_BIT_3 => ZERO32(3 downto 0),
	TRIGGER_4_BIT_SAMPLE_CLK_3 => ZERO,
	TRIGGER_4_BIT_4 => ZERO32(3 downto 0),
	TRIGGER_4_BIT_SAMPLE_CLK_4 => ZERO,
	TRIGGER_8_BIT_1 => ZERO32(7 downto 0),
	TRIGGER_8_BIT_SAMPLE_CLK_1 => ZERO,
	TRIGGER_8_BIT_2 => ZERO32(7 downto 0),
	TRIGGER_8_BIT_SAMPLE_CLK_2 => ZERO,
	TRIGGER_8_BIT_3 => ZERO32(7 downto 0),
	TRIGGER_8_BIT_SAMPLE_CLK_3 => ZERO,
	TRIGGER_8_BIT_4 => ZERO32(7 downto 0),
	TRIGGER_8_BIT_SAMPLE_CLK_4 => ZERO,
	TRIGGER_16_BIT_1 => ZERO32(15 downto 0),
	TRIGGER_16_BIT_SAMPLE_CLK_1 => ZERO,
	TRIGGER_16_BIT_2 => ZERO32(15 downto 0),
	TRIGGER_16_BIT_SAMPLE_CLK_2 => ZERO,
	TRIGGER_16_BIT_3 => ZERO32(15 downto 0),
	TRIGGER_16_BIT_SAMPLE_CLK_3 => ZERO,
	TRIGGER_16_BIT_4 => ZERO32(15 downto 0),
	TRIGGER_16_BIT_SAMPLE_CLK_4 => ZERO,
	REG250 => REG250,
	REG251 => REG251,
	TRIGGER_REARM_TOGGLE => TRIGGER_REARM_TOGGLE,
	FORCE_TRIGGER_TOGGLE => FORCE_TRIGGER_TOGGLE,
	REG250_READ => REG250_READ,
	START_CAPTURE_TOGGLE => START_CAPTURE_TOGGLE
);

DUMMY_OUT <= SERIAL_COM_2 or SERIAL_COM_1;
-- prevents pullup on these signals		


--// TEST POINTS
--TEST_POINTS(1) <= DCM_LOCKED_001;
--TEST_POINTS(2) <= DCM_LOCKED_002;
--TEST_POINTS(3) <= DCM_LOCKED_003;
--TEST_POINTS(4) <= USB_CLK60G;		-- 60 MHz input clock
--TEST_POINTS(5) <= CLK_P;			-- 90 MHz processing clock CLK_P
--TEST_POINTS(6) <= CLK_IO;			-- 40 MHz I/O clock
--TEST_POINTS(7) <= AFE_CLK64G;		-- 64 MHz Analog Front-End clock
--TEST_POINTS(8) <= ASYNC_RESET;
--TEST_POINTS(9) <= ASYNC_RESET_DCM;
--TEST_POINTS(10) <= '0';


TEST_POINTS(3 downto 1) <= TP1(3 downto 1);
TEST_POINTS(4) <= TP2(1);			
TEST_POINTS(5) <= TP2(2);			
TEST_POINTS(6) <= ASYNC_RESET;			
TEST_POINTS(7) <= SERIAL_COM_2 or SERIAL_COM_1;		
TEST_POINTS(8) <= RX_USB_BUFFER_EMPTY;
TEST_POINTS(9) <= RX_USB_SAMPLE_CLK_REQ;
TEST_POINTS(10) <= '0';
 
--// MONITORING & CONTROL -------------
--// Serial interfaces are used for module-to-module communication.
-- Do not change this section unless indicated in the comments.
-- REG255 reserved for reading 4 serial port inputs
-- These signals are pulled high (to mimic stop bit).

REG255(0) <= RX_L	;	-- port 1 left = ComBlock 40-pin interface
REG255(1) <= RX_B	;	-- port 2 bottom = Comblock 40-pin interface to external transceiver
REG255(2) <= RX_R ;	
REG255(3) <= RX_USB_BUFFER_EMPTY; 	
	-- When low, uC must read byte from USB or LAN. 
REG255(7 downto 4) <= "0000";

-- Reclock TX_USB_TOGGLE
RECLOCK_TX_USB_TOGGLE: process(CLK_P,TX_USB_TOGGLE)
begin
	if rising_edge(CLK_P) then
		TX_USB_TOGGLE_D <= TX_USB_TOGGLE;
		TX_USB_TOGGLE_D2 <= TX_USB_TOGGLE_D;
		if(TX_USB_TOGGLE_D /= TX_USB_TOGGLE_D2) then
			TX_USB_SAMPLE_CLK <= '1';
		else
			TX_USB_SAMPLE_CLK <= '0';
		end if;
	end if;
end process;


--// Atmel microcontroller interface
-- Used for monitoring and control.
-- address latch
UC_ADDR_001: process(UC_ALE, UC_FLASH_CSN, UC_AD)
begin
	if falling_edge(UC_ALE) then
		if(UC_FLASH_CSN = '1') then
			UC_ADDRESS <= UC_AD;
	  	end if;
  	end if;
end process;

RECLOCK_ADDR_D: process(ASYNC_RESET, CLK_P, UC_ADDRESS)
begin
	if(ASYNC_RESET = '1') then
		UC_ADDR_D <= (others =>'0');
		UC_ADDR_D2 <= (others =>'0');
	elsif rising_edge(CLK_P) then
		UC_ADDR_D <= UC_ADDRESS;
		UC_ADDR_D2 <= UC_ADDR_D;
	end if;
end process;

-- Immediately upon completing reading a byte from the USB or LAN, ask for the 
-- next one. 
RECLOCK_RDN_D: process(CLK_P, UC_RDN)
begin
	if rising_edge(CLK_P) then
		UC_RDN_D <= UC_RDN;
		UC_RDN_D2 <= UC_RDN_D;
		RX_USB_BUFFER_EMPTY_D <= RX_USB_BUFFER_EMPTY;
		RX_USB_BUFFER_EMPTY_D2 <= RX_USB_BUFFER_EMPTY_D;

		if (UC_RDN_D = '1') and (UC_RDN_D2 = '0') and (UC_ADDR_D2 = 254)then
			-- end of M&C byte read from Atmel uC.
			if(RX_USB_BUFFER_EMPTY_D2 = '0') then
				-- last M&C byte read was over USB. Increment USB elastic buffer read pointer
				RX_USB_SAMPLE_CLK_REQ <= '1';
			end if;
		else
			RX_USB_SAMPLE_CLK_REQ <= '0';
		end if;
	end if;
end process;

-- read
UC_READ_001: process(UC_FLASH_CSN, UC_RDN, UC_ADDRESS, SREG0, SREG1, 
	SREG2, SREG3, SREG4, REG250, REG251, REG255, RX_USB, RX_USB_BUFFER_EMPTY)
begin
	if(UC_FLASH_CSN = '1') and (UC_RDN = '0') then
		if(UC_ADDRESS(7 downto 2) = "111111") then
			if(UC_ADDRESS(1 downto 0) = "11") then
				UC_AD <= REG255;			-- reg 255. Async serial from 3 connectors
			elsif(UC_ADDRESS(1 downto 0) = "10") then
				-- byte received over USB
				UC_AD <= RX_USB;		-- reg 254 = 8-bit data from USB M&C channel
			elsif(UC_ADDRESS(1 downto 0) = "01") then
				UC_AD <= REVISION;	-- reg 253 = fpga revision
			elsif(UC_ADDRESS(1 downto 0) = "00") then
				UC_AD <= OPTION;		-- reg 252 = fpga option
			else
				UC_AD <= "ZZZZZZZZ";
			end if;
		elsif(UC_ADDRESS= 0) then
			UC_AD <= SREG0;
		elsif(UC_ADDRESS= 1) then
			UC_AD <= SREG1;
		elsif(UC_ADDRESS= 2) then
			UC_AD <= SREG2;
		elsif(UC_ADDRESS= 3) then
			UC_AD <= SREG3;
		elsif(UC_ADDRESS= 4) then
			UC_AD <= SREG4;
		elsif(UC_ADDRESS= 5) then
			UC_AD <= SREG5;
		elsif(UC_ADDRESS= 6) then
			UC_AD <= SREG6;
		elsif(UC_ADDRESS= 7) then
			UC_AD <= SREG7;
		elsif(UC_ADDRESS= 8) then
			UC_AD <= SREG8;
		elsif(UC_ADDRESS= 9) then
			UC_AD <= SREG9;
		elsif (UC_ADDRESS = 251) then
			UC_AD <= REG251;		-- reg 251 =comscope  status
		elsif (UC_ADDRESS = 250) then
			UC_AD <= REG250;		-- reg 250 = comscope data byte
		else
			UC_AD <= "ZZZZZZZZ";
		end if;
	else
		UC_AD <= "ZZZZZZZZ";
 	end if;
end process;

-- detect when address 250 is being read
REG250_READ <= '1' when (UC_FLASH_CSN = '1') and (UC_RDN = '0') and (UC_ADDRESS = 250) else '0';

-- write. 
-- Registers 0 through 15 are control registers.
UC_WRITE_001: process(ASYNC_RESET, UC_FLASH_CSN, UC_WRN, UC_ADDRESS, UC_AD)
begin
	if(ASYNC_RESET = '1') then
		-- CAREFUL!!!!!! 
		-- when the external clock comes from a bigger FPGA, the ASYNC_RESET
		-- may arrive here AFTER the Atmel uC has configured the registers.
		-- We should avoid resetting the registers.
		TX_L <= '1';	-- stop bit
		TX_B <= '1';
		TX_R <= '1';
	 elsif falling_edge(UC_WRN) then

		if (UC_ADDRESS(7 downto 4) = 0) then
			if (UC_ADDRESS(3 downto 0) = 0) then
				REG0 <= UC_AD;
			end if;
			if (UC_ADDRESS(3 downto 0) = 1) then
				REG1 <= UC_AD;
			end if;
			if (UC_ADDRESS(3 downto 0) = 2) then
				REG2 <= UC_AD;
			end if;
			if (UC_ADDRESS(3 downto 0) = 3) then
				REG3 <= UC_AD;
			end if;
			if (UC_ADDRESS(3 downto 0) = 4) then
				REG4 <= UC_AD;
			end if;
			if (UC_ADDRESS(3 downto 0) = 5) then
				REG5 <= UC_AD;
			end if;
			if (UC_ADDRESS(3 downto 0) = 6) then
				REG6 <= UC_AD;
			end if;
			if (UC_ADDRESS(3 downto 0) = 7) then
				REG7 <= UC_AD;
			end if;
			if (UC_ADDRESS(3 downto 0) = 8) then
				REG8 <= UC_AD;
			end if;
			if (UC_ADDRESS(3 downto 0) = 9) then
				REG9 <= UC_AD;
 				CONFIG_CHANGE_TOGGLE <= not CONFIG_CHANGE_TOGGLE;	
				-- Configuration change toggles upon writing the last control register
				-- This happens invisibly to the user (only the Atmel uC writes to REG9
				-- which is invible to the user and to the ComBlock Control Center).
			end if;
		end if;
		if (UC_ADDRESS = 254) then
			-- REG254 reserved for output serial
			TX_L <= UC_AD(0);
			TX_B <= UC_AD(1);
			-- right connector serial output pin depends on whether 
			-- a LAN adapter is used or not on the same connector.
			-- Be sure to disable TX_R pin at the output mux.
			TX_R <= UC_AD(2);
		end if;
		if (UC_ADDRESS = 255) then
			-- REG255 write reserved for output serial (8-bit parallel format)
			TX_USB <= UC_AD;
			TX_USB_TOGGLE <= not TX_USB_TOGGLE;
		end if;
	end if;
end process;

--// reserved COMSCOPE
UC_WRITE_002: process(UC_FLASH_CSN, UC_WRN, UC_ADDRESS, UC_AD, 
	FORCE_TRIGGER_TOGGLE, TRIGGER_REARM_TOGGLE, START_CAPTURE_TOGGLE)
begin
	if falling_edge(UC_WRN) then
		if(UC_FLASH_CSN = '1') then
			-- Reserved for ComScope			
			if (UC_ADDRESS = 237) then
				REG237 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 238) then
				REG238 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 239) then
				REG239 <= UC_AD;
			end if;
			if (UC_ADDRESS = 240) then
				REG240 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 241) then
				REG241 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 242) then
				REG242 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 243) then
				REG243 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 244) then
				REG244 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 245) then
				REG245 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 246) then
				REG246 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 247) then
				REG247 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 248) then
				REG248 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 249) then
				REG249 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 250) then 
				-- detect one-shot force trigger	
				if(UC_AD(0) = '1') then	
					FORCE_TRIGGER_TOGGLE <= not(FORCE_TRIGGER_TOGGLE);
				end if;	
				-- detect one-shot re-arm	
				if(UC_AD(1) = '1') then	
					TRIGGER_REARM_TOGGLE <= not(TRIGGER_REARM_TOGGLE);
				end if;	
				-- start capturing again
				if(UC_AD(2) = '1') then	
					START_CAPTURE_TOGGLE <= not (START_CAPTURE_TOGGLE);
				end if;	
			end if;
		end if;
	end if;
end process;

-- Reclock CONFIG_CHANGE_TOGGLE to be synchronous with processing clock CLK_P
RECLOCK_002: process(CLK_P)
begin
	if rising_edge(CLK_P) then
		CONFIG_CHANGE_TOGGLE_D <= CONFIG_CHANGE_TOGGLE;
		CONFIG_CHANGE_TOGGLE_D2 <= CONFIG_CHANGE_TOGGLE_D;
		if(CONFIG_CHANGE_TOGGLE_D /= CONFIG_CHANGE_TOGGLE_D2) then
			CONFIG_CHANGE_PULSE <= '1';
		else
			CONFIG_CHANGE_PULSE <= '0';
		end if;
	end if;
end process;


end Behavioral;
